library ac_dart.server.app;

import 'acdart_router.dart';
import 'package:atlassian_connect_config/atlassian_connect_config.dart';

import 'package:shelf/shelf.dart';
import 'dart:async';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'acdart_shelf_middleware.dart' as m show
  moustacheFile, productHostAuthenticator, sessionTokenAuthenticator,
  getSessionContext, staticFile, jwtTimingLogger;



import 'package:mojito/src/mojito_impl.dart' as m;


AcDartApp acdart(AcDartConfigFactory configFactory) =>
    new AcDartApp(configFactory);


class AcDartApp extends m.MojitoImpl {
  AcDartRouter get router => super.router;
  final AcHostService hostService;
  final AcDartConfig config;
  final Middleware atlassianHostAuthenticator;
  final Middleware addonSessionAuthenticator;
  final _MoustacheFile moustacheFile = m.moustacheFile;
  final _StaticFile staticFile = m.staticFile;
  final Middleware jwtTimingLogger = m.jwtTimingLogger;
  bool _hasStarted = false;


  AcDartApp._internal(AcDartConfig config, AcHostService hostService,
      this.atlassianHostAuthenticator, this.addonSessionAuthenticator,
      bool isDevMode)
      : this.config = config,
        this.hostService = hostService,
        super(() => new AcDartRouter(hostService, config.addonHost), true,
        isDevMode: () => isDevMode) {
    middleware.global.addMiddleware(_dodgyMiddleware);
  }

  factory AcDartApp(AcDartConfigFactory configFactory) {

    final config = configFactory.config;

    final hostService = new AcHostService(config.acHostRepo, config.addonHost);

    return new AcDartApp._internal(config, hostService,
        m.productHostAuthenticator(hostService),
        m.sessionTokenAuthenticator(hostService), configFactory.isDevelopmentMode);
  }

  // TODO: how to configure logging? Could actually move to AcDartConfig ??
//  void configureLogger() {
//  }

  Future start({int port: 9999}) async {
    if (_hasStarted) {
      return null;
    }
    _hasStarted = true;

    _initLogging();

    await super.start(port: config.serverPort);

    if (config.installAddonOnStartup) {
        _log.info('installing addon...');
        try {
          await Future.wait(_autoRegisterHostUrls.map((baseUrl) =>
              hostService.installAddonToHost(baseUrl)));
        } catch(e, stackTrace) {
          _log.warning('caught error', e, stackTrace);
        }
    }

    return null;

  }

}

// TODO: added during bitbucket testing. Evalulate need
Handler _dodgyMiddleware(Handler inner) {
  return (Request request) {
    return new Future.sync(() => inner(request)).then((Response response) {
      print(response.headers);
      final r = response.headers.containsKey('x-frame-options') ?
          response.change(headers: {
        'Access-Control-Allow-Origin': '*',
        'x-frame-options': 'ALLOW-FROM *'
      }) : response;

      final nr = r.headers.containsKey('content-encoding') ?
        new Response(r.statusCode,
          headers: {}..addAll(r.headers)..remove('content-encoding'),
          context: r.context, body: r.read())
        : r;
      print(nr);
      return nr;
    });
  };
}


void _initLogging() {
  Logger.root.level = Level.FINEST; // TODO: read from env
  Logger.root.onRecord.listen((LogRecord rec) {
    print('${rec.level.name}: ${rec.time}: ${rec.message}');
  });
}



// TODO: move into config so can auto register for non prod herokus (assuming that works)
final List<Uri> _autoRegisterHostUrls = [
  Uri.parse("http://localhost:1990/confluence"),
  Uri.parse("http://localhost:2990/jira")
];

typedef Handler _MoustacheFile(String path,
                         { Map<String, dynamic> templateVariables,
                           bool includeHeaders,
                           bool includeExtraParams});

typedef Handler _StaticFile(String path);

final Logger _log = new Logger('AcDartServer');

